/*jshint laxcomma: true, smarttabs: true, node:true, mocha: true*/
'use strict';
/**
 * @module tastypie-bookshelf/lib/fields/yesno
 * @author Eric Satterwhite
 * @since 1.0.0
 * @requires tastypie
 * @requires tastypie/lib/fields/boolean
 * @requires cage/class
 * @requires cage/string/toModule
 * @requires async
 * @requires boom
 * @requires util
 * @requires debug
 */

var tastypie     = require( 'tastypie' )
  , BooleanField = require('tastypie/lib/fields/boolean')
  , Class        = require('@fulfill/cage/class')
  , toModule     = require('@fulfill/cage/string/toModule')
  , async        = require('async')
  , boom         = require('boom')
  , util         = require('util')
  , debug        = require('debug')('memphis:fields:yesno')
  , YesNoField
  ;

  /**
   * Customer Boolean field for dealing with mySQL enum fields - Y/N.
   * Converts a Yes / No values into true / false
   * @constructor
   * @alias module:tastypie-bookshelf/lib/fields/yesno
   * @extends module:tastypie/lib/fields/boolean
   * @param {Object} [options]
   * @param {String} [options.yes='Y']
   * @param {String} [options.no='N']
   * @example var x = new require('tastypie-bookshelf/lib/fields/yesno')({yes:'yes', no:'no'});
   */
YesNoField = new Class({
    inherits: BooleanField
    ,options:{
        yes:'Y'
      , no: 'N'
    }
    ,convert: function( value ){
        return value === this.options.yes ? true : false
    }
    ,hydrate: function hydrate(bundle, cb ){
        var that = this;
        this.parent('hydrate', bundle, function( err, value){
            if( err || value == null ){
              return cb( err, null );
            };

            if( value === true ){
                return cb( null, that.options.yes)
            } else if ( value === false ){
                return cb( null, that.options.no )
            } else {
              err = boom.create(
                  400
                , util.format(
                    'invalid field values for field: "%s": %s. choose one of -  %s'
                  , that.options.name
                  , value
                  , [true, false]
                )
              )
              cb( err, null )
            }
        })
    }
});


debug('defining YesnoField as yesno')
Object.defineProperty( tastypie.fields, 'yesno', {
  get: function(){
    return YesNoField
  }
});

module.exports = YesNoField
