/*jshint laxcomma: true, smarttabs: true, node:true, mocha: true*/
'use strict';
/**
 * Allows for filtering of a date with in a spefic time period using short time codes
 * @module tastypie-bookshelf/lib/resource/filters/period
 * @author Eric Satterwhite
 * @since 1.0.0
 * @example ?foobar__period=1d
 * @example ?foobar__period=2w
 * @example ?foobar__period=3m
 * @example ?foobar__period=1y
 */
var joi = require('joi')
  , boom = require('boom')
  ;

const periodexp = /(\d+)(d|m|w|y)/;
const validator = joi
                    .array()
                    .ordered(
                        joi.number()
                        , joi.string()
                              .replace(/^y$/,'YEAR' )
                              .replace(/^m$/,'MONTH' )
                              .replace(/^w$/,'WEEK' )
                              .replace(/^d$/,'DAY' )
                              .regex(/(YEAR|MONTH|WEEK|DAY)/)
                    );

module.exports = function period(qb, field, term){
    let result;
    term = periodexp.exec( term );
    if(!term){
        throw boom.badData('period must inclue a alphanumeric time period ( 1y, 2w )')
    }

    result = validator.validate( term.slice( 1 ) );

    if( result.error ){
        throw result.error
    }

    term = null;
	return qb.whereRaw(`${field} > CURDATE() - INTERVAL ${result.value[0]} ${result.value[1]}`);
};
